require 'lib/configfile/services'
require "main/container"
require 'gdbm'
require 'GDChart'

Figlet = :FirewallFiglet

IPTABLES = "/sbin/iptables"
IPTABLESLIB = "/lib/iptables"
IPTABLES_RESTORE = "/sbin/iptables-restore"
IPTABLES_SAVE = "/sbin/iptables-save"
IPTABLESCONF_PATH = "/etc/sysconfig/"
CAT = "/bin/cat"

class FirewallFiglet < Core::Figlet
    attr_reader :cnf
    attr_reader :blacklist_cnf
    attr_reader :tables
    attr_reader :overallstats
    def initialize(tm)
	super("Firewall")
	@filenames = ["iptables"]
	@base_path = IPTABLESCONF_PATH
	@cnf = nil

	@firewall = Firewall.new("IPTABLES_RCONF", self)
	@hierarchy = @firewall

	load_statistics

	# init overall statistics
	@overallstats = RuleOverallStats.new("overallstats",@firewall)

	loadconfig

	@overallstats.redraw_stats

	main
	
	serviceblocking(tm)
		
    end
    
    def save
	saveconfig
	reloadservice
	end
    
    
    def loadconfig
	    if File.exist?(@base_path + @filenames[0])
		@cnf = @firewall.read_config(@base_path+@filenames[0])

	        @cnf.each do |table|
		    @firewall.addChild(table)
		end
		
		@firewall.read_blacklist_config()

	    end
    end

    def saveconfig
	@firewall.save_config(@base_path+@filenames[0])
    end
    
    def main
	reloadservice
    end
    
    def reloadservice
		Kernel::system_guarded("#{IPTABLES} -F")
		Kernel::system_guarded("#{IPTABLES} -X")
		Kernel::system_guarded("#{IPTABLES} -t filter -F")
		Kernel::system_guarded("#{IPTABLES} -t nat -F")
		Kernel::system_guarded("#{IPTABLES} -t mangle -F")
		Kernel::system_guarded("#{IPTABLES} -t filter -X")
		Kernel::system_guarded("#{IPTABLES} -t nat -X")
		Kernel::system_guarded("#{IPTABLES} -t mangle -X")
		Kernel::system_guarded("#{IPTABLES_RESTORE} < #{@base_path}#{@filenames[0]}")
    end

    def load
	@firewall = Firewall.new("IPTABLES_RCONF", self)
	@hierarchy = @firewall
	loadconfig
	reloadservice
	main
    end    

    def load_statistics(logrotate="")
	statfiles = ["mangle.dump", "nat.dump", "filter.dump"]
	stats_path = "/var/cache/rconf/stats/"
	statfiles.collect! {|x| x + logrotate}
	@tables = Hash.new
	@labels = Array.new
	statfiles.each do |fname|
	    if File.exist?(stats_path + fname)
    		gfile = GDBM.open stats_path + fname
	    
    		tableIndex = fname.sub(/\.dump[\.\d]*/, '')

		table_statistics_array = Array.new # array we put time series to
		tables[tableIndex] = table_statistics_array
    		gfile.sort.each do |key, val|
		    @labels << key	
		    line = Marshal.load(val)	
		    line.each do |element|
			element[1].each do |subelement|
				subelement << key
				#p subelement if tableIndex == "filter"
			end
		    end
		    table_statistics_array << line
		end
		gfile.close
	    else raise GUIException,"Hm, I have no statistics history for this time"
	    end	
	end
    end
    
    def serviceblocking(tm)
    
	# service blocking init
	@services = Core::Form.new("services",@firewall)
	tm.figlets.each do |figlet|
	    if figlet.respond_to?(:firewall_interact)
		@serviceblocking = ServiceBlocking.new("#{figlet.name}",@services)
		@serviceblocking.rules = []
		
			# looking in rules for service blocking/enable info. Format figlet.name:enable or figlet.name:block
			@rules = @firewall.ref("#{@firewall.uid}/filter/INPUT/rules")
			@rules.children.each { |rule|
			    if rule.class == Rule && rule.service_figlet
				if rule.service_figlet == figlet.name
				    @serviceblocking.rules.push(rule)
				    @serviceblocking.internet.set rule.service_enable
				end
	    		    end
			}
			@rules = @firewall.ref("#{@firewall.uid}/filter/FORWARD/rules")
			@rules.children.each { |rule|
			    if rule.class == Rule && rule.service_figlet   
				@serviceblocking.rules.push(rule) if rule.service_figlet == figlet.name
	    		    end
			}
			@rules = @firewall.ref("#{@firewall.uid}/filter/OUTPUT/rules")
			@rules.children.each { |rule|
			    if rule.class == Rule && rule.service_figlet
				@serviceblocking.rules.push(rule) if rule.service_figlet == figlet.name
	    		    end
			}
		# query service if no rules found	
		if @serviceblocking.rules.empty?
		    figlet.firewall_interact.each { |key,val|
			val.each { |rule|
			    obj = @firewall.parseLine(rule)
			    obj.service_figlet = figlet.name
			    obj.service_enable = "block"
			    obj.invisible=true
			    @serviceblocking.rules.push(obj)
			}    
		    }
		end	
		
	    end	
	end
	@action = Core::Action.new("save",@services){ |x|
    	    x.each {|obj, val|
	    
		if val == 'block' || val == 'enable'
		    obj.parent.rules.each { |rule|
			rule.service_enable=val
		    }
		end    
		obj.set(val)
	    }
	    #@firewall.print_config()
	    @firewall
	}
    end
    
end

class Firewall < Core::Notebook

    def initialize(name, residence)
	super(name, nil, nil)
	@residence = residence	

	@mangle_table = Table.new("mangle", self, nil)
	@nat_table = Table.new("nat", self, nil)
	@filter_table = Table.new("filter", self, nil)
	
	# blacklist init
	@blacklist = Blacklist.new("blacklist", self)
	
	if get_status
	    @status_str = 'Running...'
	else
	    @status_str = 'Stopped...'
	end
	Core::Message.new("Service status",self,@status_str)
#	@action = Core::Action.new("printConfig",self){print_config();self}
#	FirewallWizard.new("wizard",self)
    end

    def get_status
	fname = '/var/lock/subsys/iptables'
        File.exist?(fname)
    end	

    def read_config(file)
	tables = []
	@rule_number = 0
	@rule_attr = nil
	File.open(file,"r") do |fileconfig|
    	    fileconfig.each_line do |$_|
		if $_=~/^#/
		    @rule_attr = $_[1..-1]
		    @rule_attr.chop
		    service_figlet,service_enable = @rule_attr.split(':')
		    if service_enable == 'block'
			obj = parseLine($_) unless $_=~/^$/
			if obj.class == Rule
			    obj.service_figlet,obj.service_enable = @rule_attr.split(':') if @rule_attr
			    obj.invisible=true
    			    @rule_attr = nil
			end
		    end
		    next
		end

		obj = parseLine($_) unless $_=~/^$/

		case obj.class
		    when Table
			tables.push(obj)
		    when Rule
			obj.service_figlet,obj.service_enable = @rule_attr.split(':') if @rule_attr
			@rule_attr = nil
		end
	    end    
	end
	return tables	
    end

	def save_config(file)
		File.open(file,"w") do |fileconfig|
			children.each do |table|
				fileconfig.print table.save if table.class == Table
			end
		end
	end

#	def save_config
#		@out = `#{IPTABLES_SAVE}`
#		File.open("/tmp/iptables.conf","w") do |fileconfig|
#				fileconfig.print @out
#		end		
#	end

    
    def print_config
	print "----------PRINT CONFIG----------------------------------\n"
	children.each do |table|
	    print table.save if table.class == Table
    	end
    end

    def parseLine(line)
	case line
	when /\*(\w+)/
	    # table match
	    @tablename = $1
	    #		print "Got table #{@tablename}\n"
	    @table = lookup(@tablename)
	    return @table
	when /:([\w\d]+)\s+([\w\-]+)\s+\[(\d+):(\d+)\]/
	    # chain match
	    @chainname = $1
	    @policy = $2
	    @chain_packet_counter = $3
	    @chain_byte_counter = $4

	    #		print "Chain #{@chainname} policy #{@policy} packets #{@chain_packet_counter} bytes #{@chain_byte_counter} in #{@tablename}\n"
	    @chain = @table.lookup(@chainname)
		@Builtin_chains = ['OUTPUT','PREROUTING','POSTROUTING','INPUT','FORWARD'] 	
	    unless @chain 
			if @Builtin_chains.include?(@chainname)
				@chain = Chain.new(@chainname,@table,nil)
			else @chain = Chain.new(@chainname,@table,nil,"USERDEFINED")
			end	
	    end
	    if @chain.class == Chain
	    	@chain.policy.set @policy 
		@chain.packets.set @chain_packet_counter
		@chain.bytes.set @chain_byte_counter
	    end    
	    return @chain

#	when /\[(\d+):(\d+)\]\s+-A\s+(\w+)/
	when /\s*-A\s+([\w\d]+)/
	    # rule match
#	    @rule_packet_counter = $1
#	    @rule_byte_counter = $2
#	    @chainname =$3
	    @chainname =$1
	    @chain = @table.lookup(@chainname)
	    @rule_number+=1
		#	print "Got rule [#{@rule_packet_counter}:#{@rule_byte_counter}] to chain #{@chainname}\n"
		#	@rule = Rule.new("Rule#{@rule_number.to_s} Packets: #{@rule_packet_counter} Bytes: #{@rule_byte_counter}",@chain)
	    @rule = Rule.new(@rule_number.to_s,@chain.rules)
		@chain.rules.counter = @rule_number
#	    @rule.packets = @rule_packet_counter
#	    @rule.bytes = @rule_byte_counter
	    @remainder = $'
	    @cnt=1
	    while @remainder
		case $'
		when /-o\s([\w\d]+)/
		    @rule.out_if.set $1 if @rule.out_if
		when /-i\s([\w\d]+)/
		    @rule.in_if.set $1 if @rule.in_if
		when /\!\s-p\s(\w+)/
		    @protocolname = $1
		    @rule.proto.proto.set @protocolname
		    @rule.proto.bool.set true
		when /-p\s(\w+)/
		    @protocolname = $1
		    @rule.proto.proto.set @protocolname
		    @rule.proto.bool.set false
		when /\!\s-f\s/
		    @rule.fragmentation.set false
		when /-f\s/
		    @rule.fragmentation.set true
		when /-s\s\!\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match inverse source/netmask
		    @rule.src.ip.set $1
		    @rule.src.netmask.set $2
		    @rule.src.bool.set true
		when /-s\s\!\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match inverse source
		    @rule.src.ip.set $1
		    @rule.src.bool.set true
		when /-s\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match source/netmask
		    @rule.src.ip.set $1
		    @rule.src.netmask.set $2
		when /-s\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match source
		    @rule.src.ip.set $1
		when /-d\s\!\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match inverse destination/netmask
		    @rule.dst.ip.set $1
		    @rule.dst.netmask.set $2
		    @rule.dst.bool.set true
		when /-d\s\!\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match inverse destination
		    @rule.dst.ip.set $1
		    @rule.dst.bool.set true
		when /-d\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match destination/netmask
		    @rule.dst.ip.set $1
		    @rule.dst.netmask.set $2
		when /-d\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
		    # match destination
		    @rule.dst.ip.set $1
		when /-m\s(\w+)/
		    # module extension match
			match_parse($1,@rule,$')
		when /-j\s(\w+)/
		    # match target extension 
		    @targetname = $1
		    @rule.target_extension.jump_target.set @targetname
		    if @targetname == 'REJECT'	
			@rule.target_extension.target = REJECT_Target_Extension.new("target_extension",@rule.target_extension)
			if $' =~ /--reject-with\s([\w+-]+)/
			    @reject_type = $1
			    @rule.target_extension.target.set @reject_type
			end	
		    elsif @targetname == 'SNAT'	
			if (@chainname != 'POSTROUTING') 
			    raise "SNAT extension should be in POSTROUTING chain."
			end
			if $' =~ /--to-source\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
			    @to_source = $1
			    @rule.target_extension.target = SNAT_Target_Extension.new("target_extension",@rule.target_extension,@to_source)
			else raise "SNAT extension incomplete."	
			end
		    elsif @targetname == 'DNAT'
			if (@chainname != 'PREROUTING' && @chainname != 'OUTPUT') 
			    raise "DNAT extension should be in PREROUTING,OUTPUT chains."
			end
			if $' =~ /--to-destination\s(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/
			    @to_destination = $1
			    @rule.target_extension.target = DNAT_Target_Extension.new("target_extension",@rule.target_extension,@to_destination)
			else raise "DNAT extension incomplete."	
			end
		    elsif @targetname == 'LOG'	
				@rule.target_extension.target = LOG_Target_Extension.new("target_extension",@rule.target_extension)
				@options = $'
				if @options =~ /--log-level\s([\d]+)/
				    @rule.target_extension.target.level.set $1.to_i
				end	
				if @options =~ /--log-prefix\s([\w\d]+)/
				    @rule.target_extension.target.prefix.set $1
				end	
				if @options =~ /--log-tcp-sequence/
				    @rule.target_extension.target.tcpseq.set true
				end	
				if @options =~ /--log-tcp-options/
				    @rule.target_extension.target.tcpopt.set true
				end	
				if @options =~ /--log-ip-options/
				    @rule.target_extension.target.ipopt.set true
				end	
		    elsif @targetname == 'TTL'	
				@rule.target_extension.target = TTL_Target_Extension.new("target_extension",@rule.target_extension)
				@options = $'
				if @options =~ /--ttl-set\s([\d]+)/
				    @rule.target_extension.target.ttlset.set $1.to_i
				end	
				if @options =~ /--ttl-dec\s([\d]+)/
				    @rule.target_extension.target.ttldec.set $1.to_i
				end	
				if @options =~ /--ttl-inc\s([\d]+)/
				    @rule.target_extension.target.ttlinc.set $1.to_i
				end	
		    elsif @targetname == 'MASQUERADE'	
			if (@chainname != 'POSTROUTING') 
			    raise "MASQUERADE extension must be in POSTROUTING chain."
			end
		    end
		    
		end    
		@remainder=false unless $'
		
	    end        
	    return @rule
    	end
    end
	
	def match_parse(name,rule,str_to_match)
	    case name
	    	when "tcp" 
				@extension = TCP_Extension.new(name,rule.extension)
				@options = str_to_match
				if @options =~ /--sport\s(\d+):(\d+)/
		    		    @extension.sport.port1.set $1 ; @extension.sport.port2.set $2
				end	
				if @options =~ /--sport\s\!\s(\d+):(\d+)/
				    @extension.sport.port1.set $1 ; @extension.sport.port2.set $2
				    @extension.sport.bool.set true
				end	
				if @options =~ /--sport\s(\d+)/
				    @extension.sport.port1.set $1
				end	
				if @options =~ /--sport\s\!\s(\d+)/
				    @extension.sport.port1.set $1
		    		    @extension.sport.bool.set true
				end	
				if @options =~ /--dport\s(\d+):(\d+)/
				    @extension.dport.port1.set $1 ; @extension.dport.port2.set $2
				end	
				if @options =~ /--dport\s\!\s(\d+):(\d+)/
			    	    @extension.dport.port1.set $1 ; @extension.dport.port2.set $2
				    @extension.dport.bool.set true
				end
				if @options =~ /--dport\s(\d+)/
				    @extension.dport.port1.set $1 
				end	
				if @options =~ /--dport\s\!\s(\d+)/
			    	    @extension.dport.port1.set $1
				    @extension.dport.bool.set true
				end
				if @options =~ /--tcp-flags\s([\w+\,]+)\s([\w+\,]+)/
				    @examine_flags = $1.split(",")
				    @set_flags = $2.split(",")
			    	    @extension.tcpflags_exm.setflags(true,@examine_flags)
				    @extension.tcpflags_set.setflags(true,@set_flags)
				end
				if @options =~ /--tcp-flags\s\!\s([\w+\,]+)\s([\w+\,]+)/
				    @examine_flags = $1.split(",")
				    @set_flags = $2.split(",")
			    	    @extension.tcpflags_exm.setflags(true,@examine_flags)
				    @extension.tcpflags_set.setflags(true,@set_flags)
				    @extension.tcpflags_bool.set true
				end
				if @options =~ /\!\s--tcp-flags\s([\w+\,]+)\s([\w+\,]+)/
				    @examine_flags = $1.split(",")
				    @set_flags = $2.split(",")
			    	    @extension.tcpflags_exm.setflags(true,@examine_flags)
				    @extension.tcpflags_set.setflags(true,@set_flags)
				    @extension.tcpflags_bool.set true
				end
		    when "udp"
				@extension = UDP_Extension.new(name,rule.extension)
				@options = str_to_match
				if @options =~ /--sport\s(\d+):(\d+)/
			    	    @extension.sport.port1.set $1 ; @extension.sport.port2.set $2
				end	
				if @options =~ /--sport\s\!\s(\d+):(\d+)/
			    	    @extension.sport.port1.set $1 ; @extension.sport.port2.set $2
			    	    @extension.sport.bool.set true
				end	
				if @options =~ /--sport\s(\d+)/
			    	    @extension.sport.port1.set $1 
				end	
				if @options =~ /--sport\s\!\s(\d+)/
			    	    @extension.sport.port1.set $1 
			    	    @extension.sport.bool.set true
				end	
				if @options =~ /--dport\s(\d+):(\d+)/
				    @extension.dport.port1.set $1 ; @extension.dport.port2.set $2
				end	
				if @options =~ /--dport\s\!\s(\d+):(\d+)/
				    @extension.dport.port1.set $1 ; @extension.dport.port2.set $2
				    @extension.dport.bool.set true
				end
				if @options =~ /--dport\s(\d+)/
				    @extension.dport.port1.set $1
				end	
				if @options =~ /--dport\s\!\s(\d+)/
				    @extension.dport.port1.set $1
				    @extension.dport.bool.set true
				end
		    when "icmp"
				@extension = ICMP_Extension.new(name,rule.extension)
				if $' =~ /--icmp-type\s([\w\-]+)/
				    @extension.icmp_type.set $1
				end
		    when "limit"
				@extension = LIMIT_Extension.new(name,rule.extension)
				case str_to_match
					when /--limit\s(\d+)/
					    @limit = $1
					    @extension.limit.set @limit
				end	
		    when "mac"
				@extension = MAC_Extension.new(name,rule.extension)
				if str_to_match =~ /--mac-source\s([[:xdigit:]]{2}\:[[:xdigit:]]{2}\:[[:xdigit:]]{2}\:[[:xdigit:]]{2}\:[[:xdigit:]]{2}\:[[:xdigit:]]{2})/
		    		@extension.mac.set $1
				else raise "MAC extension incomplete."	    
				end
		    when "state"
				@extension = STATE_Extension.new(name,rule.extension)
				if str_to_match =~ /--state\s([\w+\,]+)/
				    @state = $1.split(",")
				    @extension.state.set @state
				else raise "STATE extension incomplete."	    
				end
		    when "ttl"
				@extension = TTL_Extension.new(name,rule.extension)
				if str_to_match =~ /--ttl-eq\s(\d+)/
				    @extension.ttl.set $1
				else raise "TTL extension incomplete."	    
				end
		    when "length"
				@extension = LENGTH_Extension.new(name,rule.extension)
				if str_to_match =~ /--length\s(\d+)/
				    @extension.packetlength $1
				else raise "LENGTH extension incomplete."	    
				end
		    when "owner"
				@extension = OWNER_Extension.new(name,rule.extension)
				@options = str_to_match
				if @options =~ /--uid-owner\s(\d+)/
			    	@owner = $1
				    @extension.uid.set @owner.to_i
				end	
				if @owner =~ /--gid-owner\s(\d+)/
				    @owner = $1
				    @extension.gid.set @owner.to_i
				end	
				if @owner =~ /--pid-owner\s(\d+)/
				    @owner = $1
				    @extension.pid.set @owner.to_i
				end	
				if @owner =~ /--sid-owner\s(\d+)/
				    @owner = $1
				    @extension.sid.set @owner.to_i
				end	
	    end # @extensionname
	end

    def read_blacklist_config()
	blackaddrs = Hash.new()

	@rules = self.ref("#{uid}/filter/INPUT/rules")
	@rules.children.each { |rule|
	if rule.class == Rule    
	    if rule.target_extension.jump_target.get == "DROP" && rule.proto.proto.get == "all" && rule.src.ip.get && ! rule.dst.ip.get
		blackaddrs[rule.src.ip.get] = [0,nil,false,[]] unless blackaddrs.has_key?(rule.src.ip.get)
		blackaddrs[rule.src.ip.get][0]+=1
		blackaddrs[rule.src.ip.get][1]=rule.src.netmask.get
		blackaddrs[rule.src.ip.get][2]=rule.src.bool.get
		blackaddrs[rule.src.ip.get][3].push(rule)
	    end
	end    
	}

	@rules = self.ref("#{uid}/filter/OUTPUT/rules")
	@rules.children.each { |rule|
	if rule.class == Rule    
	    if rule.target_extension.jump_target.get == "DROP" && rule.proto.proto.get == "all" && rule.dst.ip.get && ! rule.src.ip.get
		blackaddrs[rule.dst.ip.get] = [0,nil,false,[]] unless blackaddrs.has_key?(rule.dst.ip.get)
		blackaddrs[rule.dst.ip.get][0]+=1
		blackaddrs[rule.dst.ip.get][1]=rule.dst.netmask.get
		blackaddrs[rule.dst.ip.get][2]=rule.dst.bool.get
		blackaddrs[rule.dst.ip.get][3].push(rule)
	    end
	end    
	}

	@rules = self.ref("#{uid}/filter/FORWARD/rules")
	@rules.children.each { |rule|
	if rule.class == Rule    
	    if rule.target_extension.jump_target.get == "DROP" && rule.proto.proto.get == "all" && rule.dst.ip.get && ! rule.src.ip.get
		blackaddrs[rule.dst.ip.get] = [0,nil,false,[]] unless blackaddrs.has_key?(rule.dst.ip.get)
		blackaddrs[rule.dst.ip.get][0]+=1
		blackaddrs[rule.dst.ip.get][1]=rule.dst.netmask.get
		blackaddrs[rule.dst.ip.get][2]=rule.dst.bool.get
		blackaddrs[rule.dst.ip.get][3].push(rule)
	    end
	    if rule.target_extension.jump_target.get == "DROP" && rule.proto.proto.get == "all" && rule.src.ip.get && ! rule.dst.ip.get
		blackaddrs[rule.src.ip.get] = [0,nil,false,[]] unless blackaddrs.has_key?(rule.src.ip.get)
		blackaddrs[rule.src.ip.get][0]+=1
		blackaddrs[rule.src.ip.get][1]=rule.src.netmask.get
		blackaddrs[rule.src.ip.get][2]=rule.src.bool.get
		blackaddrs[rule.src.ip.get][3].push(rule)
	    end
	end    
	}

	blackaddrs.each { |key,val|
	    if val[0] == 4
		@blackip = BlackIp.new(key,@blacklist,key,val[1],val[2])
		@blackip.rules=val[3]
	    end
	}

    end


end

class Table < Core::MutableList

    def initialize(name, parent, children)
	super(name, parent) {|x| Chain===x}
	#@action = Core::Action.new("newChain",self){Chain.new("USERDEFINED",self,nil,'USERDEFINED')}
# 2.4.10 kernel	@Builtin_chains = {'mangle'=>['OUTPUT','PREROUTING'],'nat'=>['PREROUTING','OUTPUT','POSTROUTING'],'filter'=>['INPUT','FORWARD','OUTPUT']} 	
# 2.4.19 kernel
	@Builtin_chains = {'mangle'=>['OUTPUT','PREROUTING','INPUT','FORWARD','POSTROUTING'],'nat'=>['PREROUTING','OUTPUT','POSTROUTING'],'filter'=>['INPUT','FORWARD','OUTPUT']} 	
	@Builtin_chains[name].each do |chain|
	    @chain = Chain.new(chain,self,nil)
	end	
    end

    def actionAdd
      Chain.new("USERDEFINED",self,nil,'USERDEFINED')
    end

    def save
	
	@str = "*#{name}\n"
	children.each do |chain|
    	    @str += ":#{chain.name} #{chain.policy} [#{chain.packets}:#{chain.bytes}]\n" if chain.class == Chain	
	end
	children.each do |chain|
		if chain.class == Chain	
		    chain.rules.children.each do |rule|
#			@str += "[#{rule.packets}:#{rule.bytes}] -A #{chain.name}#{rule.save}\n" if rule.class == Rule
			if rule.class == Rule
			    @str += "\##{rule.service_figlet}:#{rule.service_enable}:" if rule.service_figlet
			    @str += "\n" if rule.service_enable == 'enable'
			    @str += "-A #{chain.name}#{rule.save}\n"
			end
		    end
		end
	end
	@str += "COMMIT\n"
        return @str
    end  
    
    def hint
	return "Table #{name}"
    end
end

module CustomChain
    def actionDelete
      true
    end
end

class Chain < Core::Form

    attr_reader :policy
    attr_writer :policy
    attr_reader :bytes
    attr_writer :bytes
    attr_reader :packets
    attr_writer :packets
    attr_reader :chaintype
    attr_writer :chaintype
    attr_reader :chainparent
    attr_writer :chainparent
    attr_reader :rules

    @@rule_counter = 0
    
    def initialize(name, parent, children,chaintype="BUILTIN")
	raise UIException unless Table===parent
	extend(CustomChain) if chaintype == 'USERDEFINED'
	super(name, parent, children)
	@policy = Policy.new("Policy",self)
	@policy.set "-" if chaintype == "USERDEFINED"
	@packets = Packets.new("Packets",self)
	@bytes = Bytes.new("Bytes",self)
	@chaintype = chaintype
	if name == 'USERDEFINED'
	    @chainname = Core::Leaf.new("Chain name",self)
	end
	@rules = Rules.new('rules', self)
	@chainaction = Core::Action.new("saveChain",self){ |x| saveChain_trigger(x) }
	#@chainaction = Core::Action.new("delChain",self){ |x| parent.delChild(self);parent } if chaintype == 'USERDEFINED'
    end
    
    def saveChain_trigger(x)
	x.each {|obj, val|
	    obj.set(val)
	    self.name = val if obj.name == "Chain name"
		self.label= val if obj.name == "Chain name"
#	    $stderr.puts "Setting #{obj.name}(#{obj.class}) to #{val}"
	}
	# allow to jump_target to this usedefined chain
	parent.children.each do |child|
		next if child.class != Chain
		child.rules.children.each do |rule|
			next if rule.class != Rule
			rule.target_extension.jump_target.values += ["#{self.name}"]
		end
	end
	parent			
    end
    
    def hint
	"Chain"
    end
end

class Rules < Core::TabularList
    attr_reader :counter
    attr_writer :counter
  def initialize(name, parent)
	raise UIExceptio unless Chain===parent
    super(name, parent) {|x| Rule===x}
	@counter = 0
  end

  def actionAdd()
    @counter+=1
    Rule.new(@counter.to_s, self)
  end
end

class RuleOverallStats < Core::Form
    attr_accessor :packets_lo, :packets_eth0, :packets_eth1
    attr_accessor :bytes_lo, :bytes_eth0, :bytes_eth1
    attr_accessor :times_lo, :times_eth0, :times_eth1
    def initialize(name, parent)
	super(name, parent)
	@packets_lo = Array.new
	@bytes_lo = Array.new
	@times_lo = Array.new
	@packets_eth0 = Array.new
	@bytes_eth0 = Array.new
	@times_eth0 = Array.new
	@packets_eth1 = Array.new
	@bytes_eth1 = Array.new
	@times_eth1 = Array.new
	
    end

    def glue(this, another, operation = "+")
	res = []
	((this.length > another.length)? this.length : another.length).times do |i|
    	    res << ( (this[i].nil? ? 0 : this[i]).send(operation, (another[i].nil? ? 0 : another[i])))
    	end
    	res
    end
    
    def show_image(title,labels,data)
	    gdc = GDChart.new
	    gdc.title = title
	    gdc.title_size = GDChart::GIANT
	    gdc.image_type = GDChart::PNG
	    gdc.PlotColor = 0xFF9933
	    gdc.BGColor = 0xFFFFFF
	    
	    tf = File.new("/var/cache/rconf/stats/image.png", "w")
	    gdc.out_graph(600, 250, tf, GDChart::BAR3D,data.length, labels, 1, data)
	    tf.close
	    Core::Image.new(title, self, "/var/cache/rconf/stats/image.png")
    end	    

    def redraw_stats

	["INPUT","FORWARD","OUTPUT"].each { |chain|
	@rules = self.ref("#{parent.uid}/filter/#{chain}/rules")
	@rules.children.each { |parentRule|
	
	next if parentRule.class != Rule

#	parentRule = parent
	parentChain = parentRule.parent.parent #we have Rules in-between
	parentTable = parentChain.parent
	packets = Array.new
	bytes = Array.new
	times = Array.new
	@children=[]
	    
	#move here
	iteration = 0 
	old_packets = 0
	old_bytes = 0
	
	@residence.tables[parentTable.name].each do |chains| 
	    next if chains.empty?
	    next unless chains[parentChain.name]
	    if chains[parentChain.name][parentRule.number]
		if iteration == 0
		    old_packets = chains[parentChain.name][parentRule.number][0].to_i
		    old_bytes = chains[parentChain.name][parentRule.number][1].to_i
		    iteration = 1
		else

		    p = (chains[parentChain.name][parentRule.number][0].to_i - old_packets)
		    b = (chains[parentChain.name][parentRule.number][1].to_i - old_bytes)
		    #p chains[parentChain.name][parentRule.number]
		    ts = chains[parentChain.name][parentRule.number][2].to_i
		    #puts ts
		    t = Time.at(ts).strftime("%d-%m %H:%M")
		    #puts t
		    #puts t.to_s
		    packets << (p > 0 ? p : 0)
		    bytes << (b > 0 ? b : 0)
		    times << t
		    old_packets = chains[parentChain.name][parentRule.number][0].to_i
		    old_bytes = chains[parentChain.name][parentRule.number][1].to_i
		    
		    if parentRule.out_if && parentRule.out_if.get == 'eth0'
		    	@packets_eth0 = glue(@packets_eth0,packets)
			@bytes_eth0 = glue(@bytes_eth0,bytes)
			@times_eth0 = times
		    elsif parentRule.out_if && parentRule.out_if.get == 'eth1'
		    	@packets_eth1 = glue(@packets_eth1,packets)
			@bytes_eth1 = glue(@bytes_eth1,bytes)
			@times_eth1 = times
		    elsif parentRule.out_if && parentRule.out_if.get == 'lo' 	
		    	@packets_lo = glue(@packets_lo,packets)
			@bytes_lo = glue(@bytes_lo,bytes)
			@times_lo = times
		    elsif parentRule.in_if && parentRule.in_if.get == 'eth0'
		    	@packets_eth0 = glue(@packets_eth0,packets)
			@bytes_eth0 = glue(@bytes_eth0,bytes)
			@times_eth0 = times
		    elsif parentRule.in_if && parentRule.in_if.get == 'eth1'
		    	@packets_eth1 = glue(@packets_eth1,packets)
			@bytes_eth1 = glue(@bytes_eth1,bytes)
			@times_eth1 = times
		    elsif parentRule.in_if && parentRule.in_if.get == 'lo' 	
		    	@packets_lo = glue(@packets_lo,packets)
			@bytes_lo = glue(@bytes_lo,bytes)
			@times_lo = times
		    end
		end
	    end
	end
	    
	}
	} # end each chain
	
	show_image("Bytes lo",times_lo,bytes_lo)
	show_image("Bytes eth0",times_eth0,bytes_eth0)
	show_image("Bytes eth1",times_eth1,bytes_eth1)

	Core::Action.new("stats_refresh", self) do |x|

    	x.each {|obj, val|
	    obj.set(val)
	    @weeksago = ".#{val}" if obj.name == "weeksago"
#	    $stderr.puts "Setting #{obj.name}(#{obj.class}) to #{val}"
	}
	    @weeksago = "" if ((@weeksago == ".") || (@weeksago == ".0"))
 	    @residence.load_statistics(@weeksago)
	    redraw_stats
 	    self
 	end

	Core::Leaf.new("weeksago",self)

     end
end

class RuleStats < Core::Form
    attr_accessor :packets, :bytes, :times
    def initialize(name, parent)
	super(name, parent)
	@packets = Array.new
	@bytes = Array.new
	@times = Array.new
	
	redraw_stats

    end
    
    def redraw_stats
	parentRule = parent
	parentChain = parentRule.parent.parent #we have Rules in-between
	parentTable = parentChain.parent
	packets = Array.new
	bytes = Array.new
	times = Array.new
	@children=[]
	    
	#move here
	iteration = 0 
	old_packets = 0
	old_bytes = 0
	
	@residence.tables[parentTable.name].each do |chains| 
	    next if chains.empty?
	    next unless chains[parentChain.name]
	    if chains[parentChain.name][parentRule.number]
		if iteration == 0
		    old_packets = chains[parentChain.name][parentRule.number][0].to_i
		    old_bytes = chains[parentChain.name][parentRule.number][1].to_i
		    iteration = 1
		else
		    p = (chains[parentChain.name][parentRule.number][0].to_i - old_packets)
		    b = (chains[parentChain.name][parentRule.number][1].to_i - old_bytes)
		    #p chains[parentChain.name][parentRule.number]
		    ts = chains[parentChain.name][parentRule.number][2].to_i
		    #puts ts
		    t = Time.at(ts).strftime("%d-%m %H:%M")
		    #puts t
		    #puts t.to_s
		    packets << (p > 0 ? p : 0)
		    bytes << (b > 0 ? b : 0)
		    times << t
		    old_packets = chains[parentChain.name][parentRule.number][0].to_i
		    old_bytes = chains[parentChain.name][parentRule.number][1].to_i
		end
	    end
	end
	
	data = packets
	labels = times

	gdc = GDChart.new
	gdc.title = "Past traffic via rule (packets)"
	gdc.title_size = GDChart::GIANT
	gdc.image_type = GDChart::PNG
	#gdc.ExtVolColor = [ 0xFF3399, 0xFF9933, 0xFFEE33, 0x33FF33, 0x33FFCC, 0x9966FF ]
	gdc.PlotColor = 0xFF9933
	gdc.BGColor = 0xFFFFFF

	tf = File.new("/var/cache/rconf/stats/image.png", "w")
	gdc.out_graph(600, 250, tf, GDChart::BAR3D,data.length, labels, 1, data)
	tf.close
	Core::Image.new("stat_img_packets", self, "/var/cache/rconf/stats/image.png")

	data = bytes
	labels = times

	gdc = GDChart.new
	gdc.title = "Past traffic via rule (bytes)"
	gdc.title_size = GDChart::GIANT
	gdc.image_type = GDChart::PNG
	#gdc.ExtVolColor = [ 0xFF3399, 0xFF9933, 0xFFEE33, 0x33FF33, 0x33FFCC, 0x9966FF ]
	gdc.PlotColor = 0xFFEE33
	gdc.BGColor = 0xFFFFFF
	
	tf = File.new("/var/cache/rconf/stats/image.png", "w")
	gdc.out_graph(600, 250, tf, GDChart::BAR3D,data.length, labels, 1, data)
	tf.close
	Core::Image.new("stat_img_bytes", self, "/var/cache/rconf/stats/image.png")

	Core::Action.new("stats_refresh", self) do |x|

    	x.each {|obj, val|
	    obj.set(val)
	    @weeksago = ".#{val}" if obj.name == "weeksago"
#	    $stderr.puts "Setting #{obj.name}(#{obj.class}) to #{val}"
	}
	    @weeksago = "" if ((@weeksago == ".") || (@weeksago == ".0"))
 	    @residence.load_statistics(@weeksago)
	    redraw_stats
 	    self
 	end

	Core::Leaf.new("weeksago",self)
	
     end
end

class Rule < Core::Form
    attr :packets , TRUE
    attr :bytes , TRUE  
    
    attr_accessor :proto, :src, :dst, :in_if, :out_if, :extension, :target_extension, :fragmentation, :service_attr, :service_figlet, :service_enable

    def initialize(name, parent)
	raise UIException unless Rules===parent
	super(name, parent)
	@action_container = Core::Bar.new("ruleActions",self,nil)
	def @action_container.orientation
	    true
	end	
	raise UIException unless Chain===(@chain = parent.parent)
	raise UIException unless Rules===(@rules = parent)
	@ruleaction = Core::Action.new("saveRule",@action_container){ |x| saveRule_trigger(x) }
	@ruleaction = Core::Action.new("delRule",@action_container){ |x| parent.delChild(self);parent}

	@packets = "0"
	@bytes = "0"
	@proto = Protocol.new("protocol",self)
	@src = NetworkAddress.new("source",self)
	@dst = NetworkAddress.new("destination",self)
	@in_if_chains = ['PREROUTING','FORWARD','INPUT']
	@out_if_chains = ['POSTROUTING','FORWARD','OUTPUT']
	@in_if = Interface.new("in_if",self) if @in_if_chains.include?(@chain.name)   
	@out_if = Interface.new("out_if",self) if @out_if_chains.include?(@chain.name)   
	@fragmentation = BooleanMask.new("applyFrag",self,false)
	@extension = Extension_Container.new('match_extension',self)
	@target_extension = Target_Extension_Container.new('target',self)	

	#adding statistics
	@statistics = RuleStats.new("stats", self)
    end

    def saveRule_trigger(x)
	x.find_all{|y, val| y.uid =~ /^#{self.uid}/ }.each { |obj , val| 
#	    	$stderr.puts "Setting #{obj.name}(#{obj.class}) to #{val}"
	    @jump_target = val if obj.class == Target_Selector
		if obj.class == Service
			if obj.parent.name == 'SourcePortRange'
			    @src_service,@src_name = val.split(/:/,2)
			elsif obj.parent.name == 'DestinationPortRange'	
			    @dst_service,@dst_name = val.split(/:/,2)
			end	
		end	
		if ((obj.class == Core::Port) && (obj.name == 'port1'))
			@src_port = val if obj.parent.name == 'SourcePortRange'
			@dst_port = val if obj.parent.name == 'DestinationPortRange'
		end
	}
	@src_service = "" if @src_service == 'unknown'
	@dst_service = "" if @dst_service == 'unknown'

	@target = self.ref("#{self.uid}/target").target		

	x.find_all{|y, val| y.uid =~ /^#{self.uid}/ }.each { |obj , val| 
		if ((obj.class == Core::Port) && (obj.name == 'port1') && (obj.parent.name == 'SourcePortRange'))
			if val.to_s == ""
				obj.set(@src_service)
			else
				obj.set(val)
			end	
		elsif ((obj.class == Core::Port) && (obj.name == 'port1') && (obj.parent.name == 'DestinationPortRange'))
			if val.to_s == ""
				obj.set(@dst_service)
			else
				obj.set(val)
			end	
		elsif ((obj.class == Service))
			if @src_port
				obj.set(obj.known_port?(@src_port)) if obj.parent.name == 'SourcePortRange'
			else
				obj.set(val) if obj.parent.name == 'SourcePortRange'
			end
			if @dst_port
				obj.set(obj.known_port?(@dst_port)) if obj.parent.name == 'DestinationPortRange'
			else
				obj.set(val) if obj.parent.name == 'DestinationPortRange'
			end
		else
	    	obj.set(val)
		end	
	}
	if ((@jump_target == "LOG" && @target.class != LOG_Target_Extension) || (@jump_target == "REJECT" && @target.class != REJECT_Target_Extension) || 
			(@jump_target == "SNAT" && @target.class != SNAT_Target_Extension) || (@jump_target == "DNAT" && @target.class != DNAT_Target_Extension))
	    raise GUIException,"Please specify correct options for #{@jump_target} Target"
	end
	parent
    end

    def uiDigest
      [@proto.uiDigest, @src.uiDigest, @dst.uiDigest, @target_extension.uiDigest]
    end

    def actionDelete
#      $stderr.puts "Deleting rule #{uid}"
      true
    end

    def save
	src_obj = " -s #{src.to_s}" if ((src.ip.get != "") and src.ip.get)
	dst_obj = " -d #{dst.to_s}" if ((dst.ip.get != "") and dst.ip.get)
	jump_target_obj = target_extension.jump_target.to_s if target_extension.jump_target
	in_if_obj = " -i #{in_if}" if (in_if.to_s != "")
	out_if_obj = " -o #{out_if}" if (out_if.to_s != "")
	extension_obj = ""
	extension.children.each do |child|
	    extension_obj += child.to_s if child.class != Core::Action
	end
	target_obj = target_extension.target.to_s if target_extension.target
	fragmentation_obj = " -f" if fragmentation == true
	fragmentation_obj = " ! -f" if fragmentation == false
	"#{in_if_obj}#{out_if_obj}#{proto.to_s}#{fragmentation_obj}#{src_obj}#{dst_obj}#{extension_obj}#{jump_target_obj}#{target_obj}"
    end
	
	def number
		@rules = []
		parent.children.each do |rule|
			@rules.push(rule) if rule.class == Rule
		end
	    @rules.index(self)
	end

end

class NetworkAddress < Core::Container
    attr_reader :ip
    attr_writer :ip
    attr_reader :netmask
    attr_writer :netmask
    attr_reader :bool
    attr_writer :bool
    def initialize(name, parent, children=nil,ip=nil,netmask=0,bool=false)
	# raise UIException unless Rule===parent
	super(name, parent, children)
	@bool = BooleanMask.new("not",self,bool)
	@ip = Core::IpAddr.new("ip",self,ip)
	@netmask = Netmask.new("netmask",self)
	@netmask.set netmask
    end
    def to_s
	"#{@bool.to_s}#{@ip.get}#{@netmask.to_s}"
    end

    def uiDigest
      @bool.to_s + @ip.get.to_s + ' / ' + @netmask.get.to_s
    end
end

class RawProtocol < Core::Variant
    @@Protocols = ['tcp', 'udp', 'icmp', 'all']
    def initialize(name, parent, proto="all")
	super(name,@@Protocols,parent,proto)
    end
    def to_s
	"#{get}"
    end
end

class Protocol < Core::Container
    attr_reader :proto
    attr_writer :proto
    attr_reader :bool
    attr_writer :bool
    def initialize(name, parent, proto="all", bool=false)
	raise UIException unless Rule===parent
	super(name,parent,nil)
	@bool = BooleanMask.new("notproto",self,bool)
	@proto = RawProtocol.new("proto",self,proto)
    end
    def to_s
	return " -p #{bool.to_s}#{@proto}" if @proto.to_s != "all"
	return ""
    end

    def uiDigest
      @bool.to_s + @proto.get.to_s
    end
end

class Policy < Core::Variant
    @@Targets = ['ACCEPT', 'DROP', 'QUEUE', 'RETURN','-']
    def initialize(name, parent, target="ACCEPT")
	raise UIException unless Chain===parent
	super(name,@@Targets,parent,target)
    end
    def to_s
	"#{get}"
    end
    def hint
	"Policy"
    end
end

class Service < Core::Variant
    @@Services = ConfigFile::Services::ServiceConfig.new
    def initialize(name, parent, proto)
#	raise UIException unless Chain===parent
	@services =  @@Services.collect{|obj| if obj.proto == proto then "#{obj.port}:#{obj.name}" end}
	@services.unshift("unknown:unknown")
	super(name,@services.compact!,parent)
    end
	
	def known_port?(port)
		self.values.each do |x| 
			@port,@service_name = x.split(/:/,2) 
			if @port == port 
				return "#{@port}:#{@service_name}"
			end
		end
		return "unknown:unknown"
	end
#    def to_s
#	"#{get}"
#    end
end

class MacAddress < Core::Leaf
    attr_reader :mac
    attr_writer :mac
    def initialize(name, parent, mac="")
	super(name,parent)
	@mac = mac
    end
    def to_s
	@mac
    end
    def hint
	"Mac"
    end
end

class Netmask < Core::Leaf
    def initialize(name, parent)
	super(name,parent)
    end
    def to_s
		get.to_i == 0 ? '' : "/#{get}"
    end
    def hint
	"Netmask"
    end
end

class BooleanMask < Core::Boolean
    def initialize(name, parent, bool=false)
	super(name,parent,bool)
    end
    def hint
	"Not"
    end
    def to_s
      self.get ? '! ' : ''
    end    
end

class Target_Selector < Core::Variant
    @@ValidTargets = ['ACCEPT', 'DROP', 'QUEUE', 'RETURN','LOG']
    def initialize(name, parent, target="DROP")
	raise UIException unless Target_Extension_Container===parent
	raise UIException unless Table===(@table = parent.parent.parent.parent.parent)
	raise UIException unless Chain===(@chain = parent.parent.parent.parent)
	@Targets = @@ValidTargets
	@table.children.each do |chain|
	    @Targets += [chain.name] if chain.class == Chain && chain.chaintype == 'USERDEFINED' && chain.name != @chain.name			
	end
	case @chain.name
	when 'INPUT'
	    @Targets += ['REJECT','MIRROR']
	when 'OUTPUT'
	    @Targets += ['REJECT']
	    @Targets += ['DNAT','REDIRECT'] if @table.name == 'nat'
	when 'FORWARD'
	    @Targets += ['REJECT','MIRROR']
	when 'PREROUTING'
	    @Targets += ['MIRROR']
	    @Targets += ['DNAT','REDIRECT'] if @table.name == 'nat'
	when 'POSTROUTING'
	    @Targets += ['SNAT','MASQUERADE'] if @table.name == 'nat'
	end	
	@Targets += ['MARK','TTL'] if @table.name == 'mangle'
	super(name,@Targets,parent,target)
    end
    def to_s
        " -j #{get}"
    end
    def hint
	"Target"
    end
end

class Target_Extension < Core::Container
    def initialize(name, parent)
	raise UIException unless Target_Extension_Container===parent
	super(name,parent,nil)
    end
end

class Target_Extension_Container < Core::Container
    attr_reader :jump_target
    attr_writer :jump_target
    attr_reader :target
    attr_writer :target
    def initialize(name, parent)
	raise UIException unless Rule===parent
	super(name,parent,nil)
	@jump_target = Target_Selector.new("target_selector",self)
	@action = Core::Action.new('defaulttargetOptions',self) { |x|
	    children.each do |child|
		delChild(child) if ((child.class != Target_Selector) && (child.class != Core::Action))
		@target = nil
	    end
	    x.find_all{|y, val| y.uid =~ /^#{self.uid}/ }.each {|obj , val| 
		@targetname = val if obj.name == "target_selector"
		obj.set(val)
	    }
	    
	    begin
		@target = instance_eval("#{@targetname}_Target_Extension.new('target_extension',self)")
	    rescue
    		raise GUIException,"#{@targetname} Target is not supported"
	    end

	    parent 
	}
    end

    def uiDigest
	@jump_target.get.to_s
    end
end

class REJECT_Target_Extension < Core::Variant
    @@REJECT_type = ['icmp-net-unreachable', 'icmp-port-unreachable','icmp-host-unreachable','icmp-proto-unreachable','icmp-net-prohibited','icmp-host-prohibited']
    def initialize(name, parent, reject_type="icmp-port-unreachable")
	super(name,@@REJECT_type,parent,reject_type)
    end
    def hint
	"Not"
    end
    def label
	"Reject as"
    end
    def to_s
	" --reject-with #{get}"
    end
    
end

class MARK_Target_Extension < Target_Extension
    def initialize(name, parent, set_mark=nil)
	super(name,parent)
        @set_mark = Core::HexNumber.new("set mark",self)
    end
    def hint
	return "Not"
    end
    def label
	return "Set mark"
    end
    def to_s
	return " --set-mark #{@set_mark.get.to_s}"
    end
    
end

class SNAT_Target_Extension < Target_Extension
    def initialize(name, parent, to_source=nil)
	super(name,parent)
	@to_source =  Core::IpAddr.new("to source IP",self,to_source)
    end
    def hint
	return "Not"
    end
    def label
	return "Translate to"
    end
    def to_s
	return " --to-source #{@to_source}"
    end
    
end

class DNAT_Target_Extension < Target_Extension
    def initialize(name, parent, to_destination=nil)
	super(name,parent)
	@to_destination =  Core::IpAddr.new("to destination IP",self,to_destination)
    end
    def hint
	return "Not"
    end
    def label
	return "Translate to"
    end
    def to_s
	return " --to-destination #{@to_destination}"
    end
    
end

class LOG_Target_Extension < Target_Extension 
    attr_reader :level
    attr_writer :level
    attr_reader :prefix
    attr_writer :prefix
    attr_reader :tcpseq
    attr_writer :tcpseq
    attr_reader :tcpopt
    attr_writer :tcpopt
    attr_reader :ipopt
    attr_writer :ipopt
    def initialize(name, parent)
	super(name,parent)
        @level = Core::Ranged.new("level",0..7,self)
        @prefix = Core::Leaf.new("prefix",self)
        @tcpseq = BooleanMask.new("tcpsequence",self,false)
        @tcpopt = BooleanMask.new("tcpoptions",self,false)
        @ipopt = BooleanMask.new("ipoptions",self,false)
    end
    def hint
	"Not"
    end
    def label
	"LOG Target"
    end
    def to_s
	level_str = " --log-level #{level.get.to_s}" if ((level.get != nil) and level.get)
	prefix_str = " --log-prefix #{prefix.get.to_s}" if ((prefix.get != "") and prefix.get)
	tcpseq_str = " --log-tcp-sequence" if tcpseq.to_s != ""
	tcpopt_str = " --log-tcp-options" if tcpopt.to_s != ""
	ipopt_str = " --log-ip-options" if ipopt.to_s != ""
	"#{level_str}#{prefix_str}#{tcpseq_str}#{tcpopt_str}#{ipopt_str}"
    end
    
end

class TTL_Target_Extension < Target_Extension 
    attr_accessor :ttlset
    attr_accessor :ttldec
    attr_accessor :ttlinc
    def initialize(name, parent)
	super(name,parent)
        @ttlset = Core::Number.new("ttlset",self)
        @ttldec = Core::Number.new("ttldec",self)
        @ttlinc = Core::Number.new("ttlinc",self)
    end
    def hint
	"Not"
    end
    def label
	"TTL Target"
    end
    def to_s
	ttlset_str = " --ttl-set #{ttlset.get.to_s}" if ((ttlset.get != nil) and ttlset.get)
	ttldec_str = " --ttl-dec #{ttldec.get.to_s}" if ((ttldec.get != nil) and ttldec.get)
	ttlinc_str = " --ttl-inc #{ttlinc.get.to_s}" if ((ttlinc.get != nil) and ttlinc.get)
	"#{ttlset_str}#{ttldec_str}#{ttlinc_str}"
    end
    
end

class Extension < Core::Container
    def initialize(name, parent)
	raise UIException unless Extension_Container===parent
	super(name,parent,nil)
	@action = Core::Action.new("removeMatch",self){ |x| parent.delChild(self);parent.parent}
    end
end

class Extension_Selector < Core::Variant
    @@EXTENSIONS = ['tcp','udp','icmp','mac','limit','owner','state','ttl']
    def initialize(name, parent, extension="tcp")
	raise UIException unless Extension_Container===parent
	super(name,@@EXTENSIONS,parent,extension)
	@extension = extension
    end
    def hint
	return "Not"
    end
    def to_s
	return ""
    end
    
end

class Extension_Container < Core::Container
    def initialize(name, parent)
	super(name,parent)
	raise UIException unless Rule===parent
	@extension_selector = Extension_Selector.new('extension_selector',self)
	@action = Core::Action.new('addMatch',self) { |x| addMatch_trigger(x) }
    end
    def addMatch_trigger(x)
	x.find_all{|y, val| y.uid =~ /^#{parent.uid}/ }.each {|obj , val| 
#	    			$stderr.puts "Setting #{obj.uid}(#{obj.class}) to #{val}"
	    @extensionname = val if obj.class == Extension_Selector
	    @proto = val if obj.class == RawProtocol 
	    @bool = val if obj.name == "notproto"
	    obj.set(val) 
	}

	begin
	case @extensionname
	    when "tcp" 
	    	TCP_Extension.new("match",self) 
	    when "udp"	
		UDP_Extension.new("match",self)
    	    when "icmp"	
		ICMP_Extension.new("match",self)
	    else
	        begin
    		    instance_eval("#{@extensionname.upcase}_Extension.new('#{@extensionname}',self)")
    		rescue
		    raise GUIException,"#{@extensionname} Extension is not supported"
		end
	end
	rescue NameNotUniqueException
	    raise GUIException,"You can't add this extension, because you have other incompatible extension"
	end		
	parent
    end
    def hint
	"Not"
    end
end

class STATE_Extension < Extension
    attr_reader :state
    attr_writer :state
    @@STATE_type = ['NEW', 'ESTABLISHED','RELATED','INVALID']
    def initialize(name, parent, state='NEW')
	super(name,parent)
	@state = Core::Multivariant.new(name,@@STATE_type,self,state)
    end
    def hint
	"Not"
    end
    def label
	"Connection state"
    end
    def to_s
	" -m state --state #{@state.get.join(",")}"
    end
end

class ICMP_type < Core::Variant
    @@ICMP_type = ['echo-reply','net-unreachable', 'port-unreachable','host-unreachable','proto-unreachable','net-prohibited','host-prohibited','echo-request','network-redirect','host-redirect']
    def initialize(name, parent, icmp_type="port-unreachable")
	super(name,@@ICMP_type,parent,icmp_type)
    end
    def hint
	"Not"
    end
    def to_s
	"#{get}"
    end
end

class ICMP_Extension < Extension
    attr_reader :icmp_type
    attr_writer :icmp_type
    def initialize(name, parent)
	raise UIException unless Extension_Container===parent
	raise UIException unless Table===(@table = parent.parent.parent.parent.parent)
	raise UIException unless Chain===(@chain = parent.parent.parent.parent)
	raise UIException unless Rule===(@rule = parent.parent)
	raise GUIException,"ICMP extension valid only with icmp protocol specified" unless ((@rule.proto.proto.get == 'icmp') && (@rule.proto.bool.get != true))
	super(name,parent)
        @icmp_type = ICMP_type.new("ICMP_type",self)
    end

    def hint
	return "Not"
    end
    def label
	return "ICMP Extension"
    end
    def to_s
	return " -m icmp --icmp-type #{@icmp_type}"
    end
    
end

class MAC_Extension < Extension
    attr_reader :mac
    attr_writer :mac
    attr_reader :bool
    attr_writer :bool
    def initialize(name, parent)
	raise UIException unless Extension_Container===parent
	raise UIException unless Table===(@table = parent.parent.parent.parent.parent)
	raise UIException unless Chain===(@chain = parent.parent.parent.parent)
	raise GUIException,"MAC extension valid only in PREROUTING,FORWARD,INPUT chains" unless ['PREROUTING','FORWARD','INPUT'].include?(@chain.name)
	super(name,parent)
        @mac = MacAddress.new("Mac",self)
        @bool = BooleanMask.new("not",self,false)
    end
    def hint
	"Not"
    end
    def label
	"MAC Extension"
    end
    def to_s
	" -m mac --mac-source #{@mac.get.to_s}"
    end
    
end

class LIMIT_Extension < Extension
    attr_reader :limit
    attr_writer :limit
    def initialize(name, parent)
	super(name,parent)
        @limit = Limit.new("Limit",self)
    end
    def hint
	"Not"
    end
    def label
	"LIMIT Extension"
    end
    def to_s
	" -m limit --limit #{@limit.get.to_s}"
    end
    
end

class LENGTH_Extension < Extension
    attr_reader :packetlength
    attr_writer :packetlength
    def initialize(name, parent)
	super(name,parent)
	#        @limit = Limit.new("Limit",self)
	#        @packetlength = PortRange.new("LengthRange",self,"","",false)
    end
    def hint
	"Not"
    end
    def label
	"LENGTH Extension"
    end
    def to_s
	" -m length --length #{@packetlength}"
    end
    
end


class TTL_Extension < Extension
    attr_reader :ttl
    attr_writer :ttl
    def initialize(name, parent)
		super(name,parent)
        @ttl = Core::Number.new("ttl",self)
    end
    def hint
	"Not"
    end
    def label
	"TTL Extension"
    end
    def to_s
	" -m ttl --ttl-eq #{@ttl.get.to_s}"
    end
    
end

class OWNER_Extension < Extension
    attr_reader :uid
    attr_writer :uid
    attr_reader :gid
    attr_writer :gid
    attr_reader :pid
    attr_writer :pid
    attr_reader :sid
    attr_writer :sid
    def initialize(name, parent)
	raise UIException unless Extension_Container===parent
	raise UIException unless Table===(@table = parent.parent.parent.parent.parent)
	raise UIException unless Chain===(@chain = parent.parent.parent.parent)
	raise GUIException,"OWNER extension valid only in OUTPUT chain" unless @chain.name == 'OUTPUT'
	super(name,parent)
        @uid = Core::Number.new("UID",self)
        @gid = Core::Number.new("GID",self)
        @pid = Core::Number.new("PID",self)
        @sid = Core::Number.new("SID",self)
    end
    def hint
		"Not"
    end
    def label
		"OWNER Extension"
    end
    def to_s
	uid_str = " --uid-owner #{uid.get.to_s}" if ((uid.get != "") and uid.get)
	gid_str = " --gid-owner #{gid.get.to_s}" if ((gid.get != "") and gid.get)
	pid_str = " --pid-owner #{pid.get.to_s}" if ((pid.get != "") and pid.get)
	sid_str = " --sid-owner #{sid.get.to_s}" if ((sid.get != "") and sid.get)
	ext_str = " -m owner" if (uid_str || gid_str || pid_str || sid_str)
	"#{ext_str}#{uid_str}#{gid_str}#{pid_str}#{sid_str}"
    end
    
end

class TcpFlag < Core::Boolean
    @@TCP_FLAGS = ['SYN','ACK','FIN','RST','URG','PSH','ALL','NONE']
    def initialize(name, parent, bool=false)
	super(name,parent,bool)
    end
    def hint
	return "Not"
    end
    def to_s
		@name
    end
end

class TcpFlags < Core::Container
    attr_reader :flags
    attr_writer :flags
    def initialize(name, parent)
	super(name,parent)
	@flags = TcpFlag.new('SYN',self)
	@flags = TcpFlag.new('ACK',self)
	@flags = TcpFlag.new('FIN',self)
	@flags = TcpFlag.new('RST',self)
	@flags = TcpFlag.new('URG',self)
	@flags = TcpFlag.new('PSH',self)
    end
    def hint
	return "Not"
    end
    def to_s
	@set_flags = []
	children.each do |flag|
	    @set_flags.push(flag) if flag.get == true     
	end
	@set_flags.join(",")
    end
    def setflags(bool,fl)
	fl.each { |flg| lookup(flg).set bool }
    end
end


class UDP_Extension < Extension
    attr_reader :sport
    attr_writer :sport
    attr_reader :dport
    attr_writer :dport
    def initialize(name, parent)
	raise UIException unless Extension_Container===parent
	raise UIException unless Table===(@table = parent.parent.parent.parent.parent)
	raise UIException unless Chain===(@chain = parent.parent.parent.parent)
	raise UIException unless Rule===(@rule = parent.parent)
	raise GUIException,"UDP extension valid only with udp protocol specified" unless ((@rule.proto.proto.get == 'udp') && (@rule.proto.bool.get != true))
	super(name,parent)
        @sport = PortRange.new("SourcePortRange",self,"","",false)
        @dport = PortRange.new("DestinationPortRange",self,"","",false)
    end

    def hint
	"Not"
    end
    def label
	"UDP Extension"
    end
    def to_s
	sport_str = " --sport #{sport}" if sport.to_s != ""
	dport_str = " --dport #{dport}" if dport.to_s != ""
	ext_str = " -m udp" if (sport_str || dport_str)
	"#{ext_str}#{sport_str}#{dport_str}"
    end
    
end

class TCP_Extension < Extension
    attr_accessor :sport
    attr_accessor :dport
    attr_accessor :tcpflags_exm
    attr_accessor :tcpflags_set
    attr_accessor :tcpflags_bool
    def initialize(name, parent)
	raise UIException unless Extension_Container===parent
	raise UIException unless Table===(@table = parent.parent.parent.parent.parent)
	raise UIException unless Chain===(@chain = parent.parent.parent.parent)
	raise UIException unless Rule===(@rule = parent.parent)
	raise GUIException,"TCP extension valid only with tcp protocol specified" unless ((@rule.proto.proto.get == 'tcp') && (@rule.proto.bool.get != true))
	super(name,parent)
    @sport = PortRange.new("SourcePortRange",self,"","",false)
    @dport = PortRange.new("DestinationPortRange",self,"","",false)
	@tcpflags_exm = TcpFlags.new("examine TcpFlags",self)
	@tcpflags_set = TcpFlags.new("set TcpFlags",self)
	@tcpflags_bool = BooleanMask.new("invert TcpFlags",self,false)
    end

    def hint
	return "Not"
    end
    def label
	return "TCP Extension"
    end
    def to_s
	sport_str = " --sport #{sport}" if sport.to_s != ""
	dport_str = " --dport #{dport}" if dport.to_s != ""
	tcpflags_str = " --tcp-flags #{tcpflags_bool.to_s}#{tcpflags_exm.to_s} #{tcpflags_set.to_s}" if ((tcpflags_exm.to_s != "") and tcpflags_exm)
	ext_str = " -m tcp" if (sport_str || dport_str || tcpflags_str)

	"#{ext_str}#{sport_str}#{dport_str}#{tcpflags_str}"
    end
    
end

class PortRange < Core::Container
    attr_reader :port1
    attr_writer :port1
    attr_reader :port2
    attr_writer :port2
    attr_reader :bool
    attr_writer :bool
    def initialize(name, parent, port1=nil, port2=nil, bool=false)
	super(name,parent,nil)
	@bool = BooleanMask.new("not",self,bool)
	@service = Service.new("service",self,parent.parent.parent.proto.proto.get)
	@port1=Core::Port.new("port1",self,port1)
	@port2=Core::Port.new("port2",self,port2)
    end

    def to_s
		port1_str = "#{port1.get.to_s}" if ((port1.get != "") and port1.get)
		port2_str = ":#{port2.get.to_s}" if ((port2.get != "") and port2.get)

        "#{bool.to_s}#{port1_str}#{port2_str}"
    end
    def hint
	"PortRange"
    end
    def label
	name
    end
end


class Packets < Core::Number
	attr_reader :disabled
    def initialize(name, parent, packets="0")
	disabled = false
	super(name,parent,packets)
    end
    def to_s
	"#{get}"
    end
    def hint
	"Packets"
    end
end


class Limit < Core::Number
    def initialize(name, parent, limit="0")
	super(name,parent,limit)
    end
    def to_s
	"#{get}"
    end
    def hint
	"Limit"
    end
end

class Bytes < Core::Number
    def initialize(name, parent, bytes="0")
	super(name,parent,bytes)
    end
    def to_s
	get.to_s
    end
    def hint
	"Bytes"
    end
end

class Interface < Core::Leaf
    def initialize(name, parent, interface="")
		super(name,parent)
		set(interface)
    end
    def to_s
		get.to_s
    end
    def hint
		"Interface"
    end
end

# class blacklist : blocks given ip is INPUT,OUTPUT,FORWARD chains
class Blacklist < Core::MutableList
    def initialize(name, parent)
        raise "Parent must be not nil" if parent.nil?
	super(name, parent) {|x| BlackIp===x}
	AddBlackIp.new("addBlackIp",self)
	Core::Action.new("save",self){ |x| save_trigger(x) }
    end
    def save_trigger(x)
	children.each {|child|

		if child.class == BlackIp && ! child.rules
		    # block output to ip
		    @rules = self.ref("#{parent.uid}/filter/OUTPUT/rules")
		    @rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
		    child.rules = [@rule]
			@rule.target_extension.jump_target.set "DROP"
			@rule.proto.proto.set "all"
			@rule.dst.ip.set child.blackaddr.ip.get
			@rule.dst.netmask.set child.blackaddr.netmask.get
			@rule.dst.bool.set child.blackaddr.bool.get
		    # block input from ip
		    @rules = self.ref("#{parent.uid}/filter/INPUT/rules")
		    @rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
		    child.rules.push(@rule)
			@rule.target_extension.jump_target.set "DROP"
			@rule.proto.proto.set "all"
			@rule.src.ip.set child.blackaddr.ip.get
			@rule.src.netmask.set child.blackaddr.netmask.get
			@rule.src.bool.set child.blackaddr.bool.get
		    # block forwarding from/to ip
		    @rules = self.ref("#{parent.uid}/filter/FORWARD/rules")
		    @rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
		    child.rules.push(@rule)
			@rule.target_extension.jump_target.set "DROP"
			@rule.proto.proto.set "all"
			@rule.src.ip.set child.blackaddr.ip.get
			@rule.src.netmask.set child.blackaddr.netmask.get
			@rule.src.bool.set child.blackaddr.bool.get
		    @rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
		    child.rules.push(@rule)
			@rule.target_extension.jump_target.set "DROP"
			@rule.proto.proto.set "all"
			@rule.dst.ip.set child.blackaddr.ip.get
			@rule.dst.netmask.set child.blackaddr.netmask.get
			@rule.dst.bool.set child.blackaddr.bool.get
		end
	}
	parent
    end
end

class BlackIp < Core::Form
    attr_accessor :blackaddr,:rules
    def initialize (name, parent, blackip=nil,blacknetmask=nil,blacknot=false)
	@name = name
	raise "Parent must be not nil" if parent.nil?
	super(name, parent)
	@blackaddr = NetworkAddress.new("blackip",self)
	@blackaddr.ip.set(blackip)
	@blackaddr.netmask.set(blacknetmask)
	@blackaddr.bool.set(blacknot)
	#Core::Action.new("save",self){ |x| blockthisip_trigger(x) }
    end
    def blockthisip_trigger(x)
	parent
    end
    def actionDelete
	
	# delete corresponding rules
	rules.each { |rule|
	    rule.parent.delChild(rule)
	} 

        parent.delChild(self)
	parent
    end
end

class AddBlackIp < Core::Container
    def initialize (name, parent)
        raise "Parent must be not nil" if parent.nil?
        super(name, parent)
	NetworkAddress.new("blackip",self)
	Core::Action.new("save",self){ |x| addBlackIp_trigger(x) }
    end
    def addBlackIp_trigger(x)
	x.each {|obj, val|
#	    $stderr.puts "Setting #{obj.name}(#{obj.class}) to #{val}"
    	    @blackip = val if obj.name == "ip"
    	    @blacknetmask = val if obj.name == "netmask"
    	    @blacknot = val if obj.name == "not"
	}
	BlackIp.new(@blackip,parent,@blackip,@blacknetmask,@blacknot)
    	parent
    end
end

class ServiceBlocking < Core::Container
    attr_accessor :rules, :local, :internet
    def initialize (name, parent)
        raise "Parent must be not nil" if parent.nil?
        super(name, parent)
	#@local = Core::Variant.new("local",['block','enable'],self)
	@internet = Core::Variant.new("internet",['block','enable'],self)
    end
end


class FirewallWizard < Core::Form
	def initialize(name, parent)
		super(name,parent,nil)
		@ext_dns = Core::Form.new("ext_dns_from_gate",self,nil)
			Interface.new("out_if",@ext_dns)
			Core::IpAddr.new("ip",@ext_dns,nil)
			Core::IpAddr.new("dns1",@ext_dns,nil)
			#Core::IpAddr.new("dns2",@ext_dns,nil)
		@int_cachedns = Core::Form.new("int_cachedns_from_int_network",self,nil)
			Interface.new("in_if",@int_cachedns)
			Core::IpAddr.new("ip",@int_cachedns,nil)
			Core::IpAddr.new("network",@int_cachedns,nil)
			Netmask.new("netmask",@int_cachedns)
			Core::Action.new('save_int_cachedns',@int_cachedns) { |x| int_cachedns_trigger(x) }
		@newnotsyn = Core::Form.new("newnotsyn",self,nil)
			Core::Action.new('save_newnotsyn',@newnotsyn) { |x| newnotsyn_trigger(x) }
		@forward = Core::Form.new("forwardenable",self,nil)
			Interface.new("out_if",@forward)
			Interface.new("in_if",@forward)
			Core::IpAddr.new("ip_gate",@forward,nil)
			Core::IpAddr.new("ip",@forward,nil)
			Netmask.new("netmask",@forward)
			Core::Action.new('save_forward',@forward) { |x| forward_trigger(x) }
	end
	def ext_dns_trigger(x)
	#
	# -A OUTPUT -p udp -o IF_EXT -s GATE.IP -d DNS.IP --sport 1024:65535 --dport 53 -j ACCEPT
	# -A INPUT -p udp -i IF_EXT -s DNS.IP -d GATE.IP --sport 53 --dport 1024:65535 -j ACCEPT
	#
		@cont = Core::Container.new("ruleActions",nil,nil)			
		x.each {|obj, val|
		    obj.set(val)
		    @ip = val if obj.name == "ip"
		    @dns1 = val if obj.name == "dns1"
		    #@dns2 = val if obj.name == "dns2"
		    @out_if = val if obj.name == "out_if"
		}

		@rules = self.ref("#{parent.uid}/filter/OUTPUT/rules")
		@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.proto.proto.set "udp"
			@rule.out_if.set @out_if
			@rule.src.ip.set @ip
			@rule.dst.ip.set @dns1
				@extension = UDP_Extension.new("udp",@rule.extension)
				@extension.sport.port1.set 1024
				@extension.sport.port2.set 65535
				@extension.dport.port1.set 53
		@rules = self.ref("#{parent.uid}/filter/INPUT/rules")
		@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.proto.proto.set "udp"
			@rule.in_if.set @out_if
			@rule.src.ip.set @dns1
			@rule.dst.ip.set @ip
				@extension = UDP_Extension.new("udp",@rule.extension)
				@extension.sport.port1.set 53
				@extension.dport.port1.set 1024
				@extension.dport.port2.set 65535
		Core::Message.new("Warning!",@cont,"Following rules will be added")
	@cont	
	end
	def int_cachedns_trigger(x)
	#
	# -A OUTPUT -p UDP -o $IF_INT -s GATE.IP -d NETWORK/NETMASK -m udp --sport 53 --dport 1024:65535 -j ACCEPT
	# -A OUTPUT -p TCP -o $IF_INT -s GATE.IP -d NETWORK/NETMASK -m tcp --sport 53 --dport 1024:65535 --tcp-flags ! SYN,RST,ACK SYN -j ACCEPT
	# -A INPUT -p UDP -i $IF_INT -s NETWORK/NETMASK  -d GATE.IP --sport 1024:65535 --dport 53 -j ACCEPT
	# -A INPUT -p UDP -i $IF_INT -s NETWORK/NETMASK  -d GATE.IP --sport 1024:65535 --dport 53 -j ACCEPT
	#
		@cont = Core::Container.new("ruleActions",nil,nil)			
		x.each {|obj, val|
   			$stderr.puts "Setting #{obj.uid}(#{obj.class}) to #{val}"
		    obj.set(val)
		    @ip = val if obj.name == "ip"
		    @network = val if obj.name == "network"
		    @netmask = val if obj.name == "netmask"
		    @in_if = val if obj.name == "in_if"
		}
		@rules = self.ref("#{parent.uid}/filter/OUTPUT/rules")
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.proto.proto.set "udp"
			@rule.out_if.set @in_if
			@rule.src.ip.set @ip
			@rule.dst.ip.set @network
			@rule.dst.netmask.set @netmask
				@extension = UDP_Extension.new("udp",@rule.extension)
				@extension.sport.port1.set 53
				@extension.dport.port1.set 1024
				@extension.dport.port2.set 65535
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.proto.proto.set "tcp"
			@rule.out_if.set @in_if
			@rule.src.ip.set @ip
			@rule.dst.ip.set @network
			@rule.dst.netmask.set @netmask
				@extension = TCP_Extension.new("tcp",@rule.extension)
				@extension.sport.port1.set 53
				@extension.dport.port1.set 1024
				@extension.dport.port2.set 65535
				@extension.tcpflags_bool.set true
				@extension.tcpflags_exm.setflags(true,['SYN','RST','ACK'])
				@extension.tcpflags_set.setflags(true,['SYN'])
		@rules = self.ref("#{parent.uid}/filter/INPUT/rules")
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.proto.proto.set "udp"
			@rule.in_if.set @in_if
			@rule.dst.ip.set @ip
			@rule.src.ip.set @network
			@rule.src.netmask.set @netmask
				@extension = UDP_Extension.new("udp",@rule.extension)
				@extension.sport.port1.set 1024
				@extension.sport.port2.set 65535
				@extension.dport.port1.set 53
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.proto.proto.set "tcp"
			@rule.in_if.set @in_if
			@rule.dst.ip.set @ip
			@rule.src.ip.set @network
			@rule.src.netmask.set @netmask
				@extension = TCP_Extension.new("tcp",@rule.extension)
				@extension.sport.port1.set 1024
				@extension.sport.port2.set 65535
				@extension.dport.port1.set 53
		Core::Message.new("Warning!",@cont,"Following rules will be added")
	@cont	
	end
	def forward_trigger(x)
	#
	# -A FORWARD -i $IF_EXT -m state --state ESTABLISHED,RELATED -j ACCEPT
	# -t nat -A POSTROUTING -o $IF_EXT -s IP -j SNAT --to-source $GATE.IP
	# -A FORWARD -i $IF_INT -o $IF_EXT -s IP -j ACCEPT
	#
		@cont = Core::Container.new("ruleActions",nil,nil)			
		x.each {|obj, val|
		    obj.set(val)
		    @ip = val if obj.name == "ip"
		    @ip_gate = val if obj.name == "ip_gate"
		    @netmask = val if obj.name == "netmask"
		    @int_if = val if obj.name == "in_if"
		    @ext_if = val if obj.name == "out_if"
		}
		@rules = self.ref("#{parent.uid}/filter/FORWARD/rules")
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.in_if.set @ext_if
				@extension = STATE_Extension.new("state",@rule.extension)
				@extension.state.set "ESTABLISHED"
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.in_if.set @ext_if
				@extension = STATE_Extension.new("state",@rule.extension)
				@extension.state.set "RELATED"
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "ACCEPT"
			@rule.in_if.set @int_if
			@rule.out_if.set @ext_if
			@rule.src.ip.set @ip
			@rule.src.netmask.set @netmask
		@rules = self.ref("#{parent.uid}/nat/POSTROUTING/rules")
			@rules.counter+=1
		    @rule = Rule.new(@rules.counter.to_s, @rules)
			@rule.target_extension.jump_target.set "SNAT"
			@rule.out_if.set @ext_if
			@rule.src.ip.set @ip
			@rule.src.netmask.set @netmask
			@rule.target_extension.target = SNAT_Target_Extension.new("target_extension",@rule.target_extension,@ip_gate)
		Core::Message.new("Warning!",@cont,"Following rules will be added")
	@cont	
	end	
end

require '<ui/firewall>'
include Plugin::Ui_firewall
